Ext.define("mx.collections.IList", function(IList) {
    /* ////////////////////////////////////////////////////////////////////////////////
//
//  ADOBE SYSTEMS INCORPORATED
//  Copyright 2005-2007 Adobe Systems Incorporated
//  All Rights Reserved.
//
//  NOTICE: Adobe permits you to use, modify, and distribute this file
//  in accordance with the terms of the license agreement accompanying it.
//
////////////////////////////////////////////////////////////////////////////////

package mx.collections 
{
    
import flash.events.IEventDispatcher;
import mx.events.CollectionEvent;

/**
 *  Dispatched when the IList has been updated in some way.
 *
 *  @eventType mx.events.CollectionEvent.COLLECTION_CHANGE
 * /
[Event(name="collectionChange", type="mx.events.CollectionEvent")]

/**
 *  A collection of items organized in an ordinal fashion.  
 *  Provides access and manipulation methods based on index.  
 *  
 *  <p>An <code>IList</code> may be a view onto data
 *  that has been retrieved from a  remote location.  
 *  When writing for a collection that may be remote,
 *  it is important to handle the case where data
 *  may not yet be available, which is indicated
 *  by the  <code>ItemPendingError</code>.</p>
 *  
 *  <p>The <code>ICollectionView</code> is an alternative
 *  to the <code>IList</code>.</p>
 *
 *  @see mx.collections.errors.ItemPendingError
 *  @see mx.collections.ICollectionView
 *  @see mx.collections.ListCollectionView
 * /
public interface IList extends IEventDispatcher
{
    //--------------------------------------------------------------------------
    //
    //  Properties
    //
    //--------------------------------------------------------------------------

    //----------------------------------
    //  length
    //----------------------------------

    /**
     *  The number of items in this collection. 
     *  0 means no items while -1 means the length is unknown.
     * <p><b>Note:</b> In Jangaroo, <code>IList</code> implementations extend <code>Array</code>, so
     * we had to change the signature to return <code>int</code>, thus "unknown" -1 is not supported.</p>
     * /
    function get length():uint;

    //--------------------------------------------------------------------------
    //
    //  Methods
    //
    //--------------------------------------------------------------------------

    /**
     *  Adds the specified item to the end of the list.
     *  Equivalent to <code>addItemAt(item, length)</code>.
     *
     *  @param item The item to add.
     * /
    function addItem(item:Object):void;
    
    /**
     *  Adds the item at the specified index.  
     *  The index of any item greater than the index of the added item is increased by one.  
     *  If the the specified index is less than zero or greater than the length
     *  of the list, a RangeError is thrown.
     * 
     *  @param item The item to place at the index.
     *
     *  @param index The index at which to place the item.
     *
     *  @throws RangeError if index is less than 0 or greater than the length of the list. 
     * /
    function addItemAt(item:Object, index:int):void;
    
    /**
     *  Gets the item at the specified index.
     * 
     *  @param index The index in the list from which to retrieve the item.
     *
     *  @param prefetch An <code>int</code> indicating both the direction
     *  and number of items to fetch during the request if the item is
     *  not local.
     *
     *  @return The item at that index, or <code>null</code> if there is none.
     *
     *  @throws mx.collections.errors.ItemPendingError if the data for that index needs to be 
     *  loaded from a remote location.
     *
     *  @throws RangeError if <code>index &lt; 0</code>
     *  or <code>index >= length</code>.
     * /
    function getItemAt(index:int, prefetch:int = 0):Object;
    
    /**
     *  Returns the index of the item if it is in the list such that
     *  getItemAt(index) == item.
     * 
     *  <p>Note: unlike <code>IViewCursor.find<i>xxx</i>()</code> methods,
     *  The <code>getItemIndex()</code> method cannot take a parameter with 
     *  only a subset of the fields in the item being serched for; 
     *  this method always searches for an item that exactly matches
     *  the input parameter.</p>
     * 
     *  @param item The item to find.
     *
     *  @return The index of the item, or -1 if the item is not in the list.
     * /
    function getItemIndex(item:Object):int;
    
    /**
     *  Notifies the view that an item has been updated.  
     *  This is useful if the contents of the view do not implement 
     *  <code>IEventDispatcher</code> and dispatches a 
     *  <code>PropertyChangeEvent</code>.  
     *  If a property is specified the view may be able to optimize its 
     *  notification mechanism.
     *  Otherwise it may choose to simply refresh the whole view.
     *
     *  @param item The item within the view that was updated.
     *
     *  @param property The name of the property that was updated.
     *
     *  @param oldValue The old value of that property. (If property was null,
     *  this can be the old value of the item.)
     *
     *  @param newValue The new value of that property. (If property was null,
     *  there's no need to specify this as the item is assumed to be
     *  the new value.)
     *
     *  @see mx.events.CollectionEvent
     *  @see mx.events.PropertyChangeEvent
     * /
    function itemUpdated(item:Object, property:Object = null, 
                         oldValue:Object = null, 
                         newValue:Object = null):void;

    /** 
     *  Removes all items from the list.
     *
     *  <p>If any item is not local and an asynchronous operation must be
     *  performed, an <code>ItemPendingError</code> will be thrown.</p>
     *
     *  <p>See the ItemPendingError documentation as well as
     *  the collections documentation for more information
     *   on using the <code>ItemPendingError</code>.</p>
     * /
    function removeAll():void;

    /**
     *  Removes the item at the specified index and returns it.  
     *  Any items that were after this index are now one index earlier.
     *
     *  @param index The index from which to remove the item.
     *
     *  @return The item that was removed.
     *
     *  @throws RangeError is index is less than 0 or greater than length. 
     * /
    function removeItemAt(index:int):Object;
    
    /**
     *  Places the item at the specified index.  
     *  If an item was already at that index the new item will replace it
     *  and it will be returned.
     *
     *  @param item The new item to be placed at the specified index.
     *
     *  @param index The index at which to place the item.
     *
     *  @return The item that was replaced, or <code>null</code> if none.
     *
     *  @throws RangeError if index is less than 0 or greater than length.
     * /
    function setItemAt(item:Object, index:int):Object;
    
    /**
     *  Returns an Array that is populated in the same order as the IList
     *  implementation.
     *  This method can throw an ItemPendingError.
     *
     *  @return The array.
     *  
     *  @throws mx.collections.errors.ItemPendingError If the data is not yet completely loaded
     *  from a remote location.
     * / 
    function toArray():Array;
}

}

============================================== Jangaroo part ==============================================*/
    return {
        mixins: [
            "flash.events.IEventDispatcher"
        ],
        requires: [
            "flash.events.IEventDispatcher"
        ]
    };
});

Ext.define("mx.collections.ListCollectionView", function(ListCollectionView) {
    /* ////////////////////////////////////////////////////////////////////////////////
//
//  ADOBE SYSTEMS INCORPORATED
//  Copyright 2005-2007 Adobe Systems Incorporated
//  All Rights Reserved.
//
//  NOTICE: Adobe permits you to use, modify, and distribute this file
//  in accordance with the terms of the license agreement accompanying it.
//
////////////////////////////////////////////////////////////////////////////////

package mx.collections
{

import flash.events.Event;

/**
 *  Dispatched when the ICollectionView has been updated in some way.
 *
 *  @eventType mx.events.CollectionEvent.COLLECTION_CHANGE
 * /
[Event(name="collectionChange", type="mx.events.CollectionEvent")]

/**
 * The ListCollectionView class adds the properties and methods of the
 * <code>ICollectionView</code> interface to an object that conforms to the
 * <code>IList</code> interface. As a result, you can pass an object of this class
 * to anything that requires an <code>IList</code> or <code>ICollectionView</code>.
 *
 * <p>This class also lets you use [ ] array notation
 * to access the <code>getItemAt()</code> and <code>setItemAt()</code> methods.
 * If you use code such as <code>myListCollectionView[index]</code>
 * Flex calls the <code>myListCollectionView</code> object's
 * <code>getItemAt()</code> or <code>setItemAt()</code> method.</p>
 * 
 * @mxml
 *
 *  <p>The <code>&lt;mx:ListCollectionView&gt;</code> has the following attributes,
 *  which all of its subclasses inherit:</p>
 *
 *  <pre>
 *  &lt;mx:ListCollectionView
 *  <b>Properties</b>
 *  filterFunction="null"
 *  list="null"
 *  sort="null"
 *  <b>Events</b>
 *  collectionChange="<i>No default</i>"
 *  /&gt;
 *  </pre>
 * /
public class ListCollectionView 
       implements IList
{
    ////////////////////////////////////////////////////////////////////////////////
//
//  ADOBE SYSTEMS INCORPORATED
//  Copyright 2005-2007 Adobe Systems Incorporated
//  All Rights Reserved.
//
//  NOTICE: Adobe permits you to use, modify, and distribute this file
//  in accordance with the terms of the license agreement accompanying it.
//
////////////////////////////////////////////////////////////////////////////////

import mx.core.mx_internal;

/**
 *  @private
 *  Version string for this class.
 * /
mx_internal static const VERSION:String = "3.3.0.4852";*/
    
    /*

  public*/
    function ListCollectionView$() {
        /*
    super()*/
        
    }
    /*

  public native function get length():uint;
  public native function set length(value:uint):void;

  public*/
    function addItem(item) /*:Object*/
    /*:void*/
    {
        throw new AS3.Error("not implemented!");
    }
    /*

  public*/
    function addItemAt(item, /*:Object*/
    index) /*:int*/
    /*:void*/
    {
        throw new AS3.Error("not implemented!");
    }
    /*

  public*/
    function getItemAt(index, /*:int*/
    prefetch) /*:int = 0*/
    /*:Object*/
    {
        if (arguments.length <= 1)  {
            prefetch = 0;
        }
        
        throw new AS3.Error("not implemented!");
    }
    /*

  public*/
    function getItemIndex(item) /*:Object*/
    /*:int*/
    {
        throw new AS3.Error("not implemented!");
    }
    /*

  public*/
    function itemUpdated(item, /*:Object*/
    property, /*:Object = null*/
    oldValue, /*:Object = null*/
    newValue) /*:Object = null*/
    /*:void*/
    {
        switch (Math.max(arguments.length, 1)) {
            case 1:
                property = null;
            case 2:
                oldValue = null;
            case 3:
                newValue = null;
        }
        throw new AS3.Error("not implemented!");
    }
    /*

  public*/
    function removeAll() /*:void*/
    {
        throw new AS3.Error("not implemented!");
    }
    /*

  public*/
    function removeItemAt(index) /*:int*/
    /*:Object*/
    {
        throw new AS3.Error("not implemented!");
    }
    /*

  public*/
    function setItemAt(item, /*:Object*/
    index) /*:int*/
    /*:Object*/
    {
        throw new AS3.Error("not implemented!");
    }
    /*

  public*/
    function toArray() /*:Array*/
    {
        throw new AS3.Error("not implemented!");
    }
    /*

  public*/
    function addEventListener(type, /*:String*/
    listener, /*:Function*/
    useCapture, /*:Boolean = false*/
    priority, /*:int = 0*/
    useWeakReference) /*:Boolean = false*/
    /*:void*/
    {
        switch (Math.max(arguments.length, 2)) {
            case 2:
                useCapture = false;
            case 3:
                priority = 0;
            case 4:
                useWeakReference = false;
        }
        throw new AS3.Error("not implemented!");
    }
    /*

  public*/
    function dispatchEvent(event) /*:Event*/
    /*:Boolean*/
    {
        throw new AS3.Error("not implemented!");
    }
    /*

  public*/
    function hasEventListener(type) /*:String*/
    /*:Boolean*/
    {
        throw new AS3.Error("not implemented!");
    }
    /*

  public*/
    function removeEventListener(type, /*:String*/
    listener, /*:Function*/
    useCapture) /*:Boolean = false*/
    /*:void*/
    {
        if (arguments.length <= 2)  {
            useCapture = false;
        }
        
        throw new AS3.Error("not implemented!");
    }
    /*

  public*/
    function willTrigger(type) /*:String*/
    /*:Boolean*/
    {
        throw new AS3.Error("not implemented!");
    }
    /*
}
}

============================================== Jangaroo part ==============================================*/
    return {
        mixins: [
            "mx.collections.IList"
        ],
        metadata: {
            "": [
                "Event",
                [
                    "name",
                    "collectionChange",
                    "type",
                    "mx.events.CollectionEvent"
                ]
            ]
        },
        constructor: ListCollectionView$,
        addItem: addItem,
        addItemAt: addItemAt,
        getItemAt: getItemAt,
        getItemIndex: getItemIndex,
        itemUpdated: itemUpdated,
        removeAll: removeAll,
        removeItemAt: removeItemAt,
        setItemAt: setItemAt,
        toArray: toArray,
        addEventListener: addEventListener,
        dispatchEvent: dispatchEvent,
        hasEventListener: hasEventListener,
        removeEventListener: removeEventListener,
        willTrigger: willTrigger,
        statics: {
            VERSION: "3.3.0.4852"
        },
        requires: [
            "AS3.Error",
            "mx.collections.IList"
        ]
    };
});

Ext.define("mx.collections.ArrayCollection", function(ArrayCollection) {
    /* ////////////////////////////////////////////////////////////////////////////////
//
//  ADOBE SYSTEMS INCORPORATED
//  Copyright 2005-2007 Adobe Systems Incorporated
//  All Rights Reserved.
//
//  NOTICE: Adobe permits you to use, modify, and distribute this file
//  in accordance with the terms of the license agreement accompanying it.
//
////////////////////////////////////////////////////////////////////////////////

package mx.collections
{

/**
 *  The ArrayCollection class is a wrapper class that exposes an Array as
 *  a collection that can be accessed and manipulated using the methods
 *  and properties of the <code>ICollectionView</code> or <code>IList</code>
 *  interfaces. Operations on a ArrayCollection instance modify the data source;
 *  for example, if you use the <code>removeItemAt()</code> method on an
 *  ArrayCollection, you remove the item from the underlying Array.
 *
 *  @mxml
 *
 *  <p>The <code>&lt;mx:ArrayCollection&gt;</code> tag inherits all the attributes of its
 *  superclass, and adds the following attributes:</p>
 *
 *  <pre>
 *  &lt;mx:ArrayCollection
 *  <b>Properties</b>
 *  source="null"
 *  /&gt;
 *  </pre>
 *
 *  @example The following code creates a simple ArrayCollection object that
 *  accesses and manipulates an array with a single Object element.
 *  It retrieves the element using the IList interface <code>getItemAt</code>
 *  method and an IViewCursor object that it obtains using the ICollectionView
 *  <code>createCursor</code> method.
 *  <pre>
 *  var myCollection:ArrayCollection = new ArrayCollection([ { first: 'Matt', last: 'Matthews' } ]);
 *  var myCursor:IViewCursor = myCollection.createCursor();
 *  var firstItem:Object = myCollection.getItemAt(0);
 *  var firstItemFromCursor:Object = myCursor.current;
 *  if (firstItem == firstItemFromCursor)
 *        doCelebration();
 *  </pre>
 * /
public class ArrayCollection extends ListCollectionView
{
  ////////////////////////////////////////////////////////////////////////////////
//
//  ADOBE SYSTEMS INCORPORATED
//  Copyright 2005-2007 Adobe Systems Incorporated
//  All Rights Reserved.
//
//  NOTICE: Adobe permits you to use, modify, and distribute this file
//  in accordance with the terms of the license agreement accompanying it.
//
////////////////////////////////////////////////////////////////////////////////

import mx.core.mx_internal;

/**
 *  @private
 *  Version string for this class.
 * /
mx_internal static const VERSION:String = "3.3.0.4852";*/
    
    /*

  //noinspection JSMismatchedCollectionQueryUpdate
  private static const*/
    var ARRAY_PROTOTYPE$static;
    /*:Array*/
    /* =*/
    function ARRAY_PROTOTYPE$static_() {
        ARRAY_PROTOTYPE$static = (Array['prototype']);
    }
    
    /*

  //--------------------------------------------------------------------------
  //
  //  Constructor
  //
  //--------------------------------------------------------------------------

  /**
   *  Constructor.
   *
   *  <p>Creates a new ArrayCollection using the specified source array.
   *  If no array is specified an empty array will be used.</p>
   * /
  public*/
    function ArrayCollection$(source) /*:Array = undefined*/
    {
        mx.collections.ListCollectionView.prototype.constructor.call(this);
        if (source) {
            for (var i = /*:uint*/
                0; i < source.length; ++i) {
                this[i] = source[i];
            }
            this.length = source.length;
        } else {
            this.length = 0;
        }
    }
    /*

  //--------------------------------------------------------------------------
  //
  // IList Methods
  //
  //--------------------------------------------------------------------------

  /**
   * @inheritDoc
   * /
  override public*/
    function getItemAt(index, /*:int*/
    prefetch) /*:int = 0*/
    /*:Object*/
    {
        if (arguments.length <= 1)  {
            prefetch = 0;
        }
        
        if (index < 0 || index >= this.length) {
            throw new /*Range*/
            AS3.Error("[collections] outOfBounds: " + index);
        }
        return this[index];
    }
    /*

  /**
   * @inheritDoc
   * /
  override public*/
    function addItem(item) /*:Object*/
    /*:void*/
    {
        this[this.length++] = item;
    }
    /*

  /**
   * @inheritDoc
   * /
  override public*/
    function toArray() /*:Array*/
    {
        return ARRAY_PROTOTYPE$static['slice'].call(this);
    }
    /*

  /**
   * @inheritDoc
   * /
  override public*/
    function getItemIndex(item) /*:Object*/
    /*:int*/
    {
        return ARRAY_PROTOTYPE$static['indexOf'].call(this, item);
    }
    /*

  /**
   * @inheritDoc
   * /
  override public*/
    function removeAll() /*:void*/
    {
        this.length = 0;
    }
    /*

  /**
   * @inheritDoc
   * /
  override public*/
    function setItemAt(item, /*:Object*/
    index) /*:int*/
    /*:Object*/
    {
        var oldItem = /*:Object*/
            this.getItemAt(index);
        this[index] = item;
        return oldItem;
    }
    /*

}
}

============================================== Jangaroo part ==============================================*/
    return {
        extend: "mx.collections.ListCollectionView",
        constructor: ArrayCollection$,
        getItemAt: getItemAt,
        addItem: addItem,
        toArray: toArray,
        getItemIndex: getItemIndex,
        removeAll: removeAll,
        setItemAt: setItemAt,
        statics: {
            VERSION: "3.3.0.4852",
            ARRAY_PROTOTYPE: undefined,
            __initStatics__: function() {
                ARRAY_PROTOTYPE$static_();
            }
        },
        requires: [
            "AS3.Error",
            "mx.collections.IList",
            "mx.collections.ListCollectionView"
        ]
    };
});

Ext.define("mx.core.mx_internal", function(mx_internal) {
    /* ////////////////////////////////////////////////////////////////////////////////
//
//  ADOBE SYSTEMS INCORPORATED
//  Copyright 2005-2006 Adobe Systems Incorporated
//  All Rights Reserved.
//
//  NOTICE: Adobe permits you to use, modify, and distribute this file
//  in accordance with the terms of the license agreement accompanying it.
//
////////////////////////////////////////////////////////////////////////////////

package mx.core
{*/
    /**
 *  This namespace is used for undocumented APIs -- usually implementation
 *  details -- which can't be private because they need to visible
 *  to other classes.
 *  APIs in this namespace are completely unsupported and are likely to
 *  change in future versions of Flex.
 *  
 *  @langversion 3.0
 *  @playerversion Flash 9
 *  @playerversion AIR 1.1
 *  @productversion Flex 3
 */
    "public namespace mx_internal" , "http://www.adobe.com/2006/flex/mx/internal" , [];
    /*
}

============================================== Jangaroo part ==============================================*/
    return {
        __factory__: mx_internal
    };
});

Ext.define("mx.events.CollectionEvent", function(CollectionEvent) {
    /* ////////////////////////////////////////////////////////////////////////////////
//
//  ADOBE SYSTEMS INCORPORATED
//  Copyright 2005-2006 Adobe Systems Incorporated
//  All Rights Reserved.
//
//  NOTICE: Adobe permits you to use, modify, and distribute this file
//  in accordance with the terms of the license agreement accompanying it.
//
////////////////////////////////////////////////////////////////////////////////

package mx.events
{

import flash.events.Event;

/**
 *  The mx.events.CollectionEvent class represents an event that is  
 *  dispatched when the associated collection changes.
 *
 *  @see FlexEvent#CURSOR_UPDATE
 * /
public class CollectionEvent extends Event
{
	////////////////////////////////////////////////////////////////////////////////
//
//  ADOBE SYSTEMS INCORPORATED
//  Copyright 2005-2007 Adobe Systems Incorporated
//  All Rights Reserved.
//
//  NOTICE: Adobe permits you to use, modify, and distribute this file
//  in accordance with the terms of the license agreement accompanying it.
//
////////////////////////////////////////////////////////////////////////////////

import mx.core.mx_internal;

/**
 *  @private
 *  Version string for this class.
 * /
mx_internal static const VERSION:String = "3.3.0.4852";*/
    
    /*

    //--------------------------------------------------------------------------
    //
    //  Class constants
    //
    //--------------------------------------------------------------------------

    /**
     *  The CollectionEvent.COLLECTION_CHANGE constant defines the value of the
     *  <code>type</code> property of the event object for an event that is
     *  dispatched when a collection has changed.
     *
     *  <p>The properties of the event object have the following values.
     *  Not all properties are meaningful for all kinds of events.
	 *  See the detailed property descriptions for more information.</p>
     *  <table class="innertable">
     *     <tr><th>Property</th><th>Value</th></tr>
     *     <tr><td><code>bubbles</code></td><td>false</td></tr>
     *     <tr><td><code>cancelable</code></td><td>false</td></tr>
     *     <tr><td><code>currentTarget</code></td><td>The Object that defines the
     *       event listener that handles the event. For example, if you use
     *       <code>myButton.addEventListener()</code> to register an event listener,
     *       myButton is the value of the <code>currentTarget</code>. </td></tr>
     *     <tr><td><code>items</code></td><td>An Array of objects with
     *       information about the items affected by the event.
	 * 		 The contents of this field depend on the event kind;
	 *       for details see the <code>items</code> property</td></tr>
     *     <tr><td><code>kind</code></td><td>The kind of event.
     *       The valid values are defined in the CollectionEventKind 
	 *       class as constants.</td></tr>
     *     <tr><td><code>location</code></td><td>Location within the target collection
     *         of the item(s) specified in the <code>items</code> property.</td></tr>
     *     <tr><td><code>oldLocation</code></td><td>the previous location in the collection
     *         of the item specified in the <code>items</code> property.</td></tr>
     *     <tr><td><code>target</code></td><td>The Object that dispatched the event;
     *       it is not always the Object listening for the event.
     *       Use the <code>currentTarget</code> property to always access the
     *       Object listening for the event.</td></tr>
     *     <tr><td><code>type</code></td><td>CollectionEvent.COLLECTION_CHANGE</td></tr>
     *  </table>
     *
     *  @eventType collectionChange
     * /
    public static const COLLECTION_CHANGE:String = "collectionChange";

    //--------------------------------------------------------------------------
    //
    // Constructor
    //
    //--------------------------------------------------------------------------

    /**
     *  Constructor.
	 *
     *  @param type The event type; indicates the action that triggered the event.
     *
     *  @param bubbles Specifies whether the event can bubble
     *  up the display list hierarchy.
     *
     *  @param cancelable Specifies whether the behavior
     *  associated with the event can be prevented.
     *
     *  @param kind Indicates the kind of event that occured.
     *  The parameter value can be one of the values in the CollectionEventKind 
	 *  class, or <code>null</code>, which indicates that the kind is unknown.
     *
     *  @param location When the <code>kind</code> is
     *  <code>CollectionEventKind.ADD</code>,
     *  <code>CollectionEventKind.MOVE</code>,
     *  <code>CollectionEventKind.REMOVE</code>, or
     *  <code>CollectionEventKind.REPLACE</code>,
     *  this value indicates at what location the item(s) specified
     *  in the <code>items property</code> can be found
     *  within the target collection.
     *
     *  @param oldLocation When the <code>kind</code> is
     *  <code>CollectionEventKind.MOVE</code>, this value indicates
     *  the old location within the target collection
     *  of the item(s) specified in the <code>items</code> property.
      *
     *  @param items Array of objects with information about the items 
	 *  affected by the event, as described in the <code>items</code> property.
     *  When the <code>kind</code> is <code>CollectionEventKind.REFRESH</code>
     *  or <code>CollectionEventKind.RESET</code>, this Array has zero length.
     * /
    public*/
    function CollectionEvent$(type, /*:String*/
    bubbles, /*:Boolean = false*/
    cancelable, /*:Boolean = false*/
    kind, /*:String = null*/
    location, /*:int = -1*/
    oldLocation, /*:int = -1*/
    items) /*:Array = null*/
    {
        switch (Math.max(arguments.length, 1)) {
            case 1:
                bubbles = false;
            case 2:
                cancelable = false;
            case 3:
                kind = null;
            case 4:
                location = -1;
            case 5:
                oldLocation = -1;
            case 6:
                items = null;
        }
        flash.events.Event.prototype.constructor.call(this, type, bubbles, cancelable);
        this.kind = kind;
        this.location = location;
        this.oldLocation = oldLocation;
        this.items = items ? items : [];
    }
    /*

    //--------------------------------------------------------------------------
    //
    // Properties
    //
    //--------------------------------------------------------------------------

	//----------------------------------
	//  kind
	//----------------------------------

    /**
     *  Indicates the kind of event that occurred.
     *  The property value can be one of the values in the 
	 *  CollectionEventKind class, 
	 *  or <code>null</code>, which indicates that the kind is unknown.
	 * 
     *  @default null
	 * 
	 *  @see CollectionEventKind
     * /
    public var kind:String;

	//----------------------------------
	//  items
	//----------------------------------

    /**
	 *  When the <code>kind</code> is <code>CollectionEventKind.ADD</code>
     *  or <code>CollectionEventKind.REMOVE</code> the <code>items</code> property
     *  is an Array of added/removed items.
	 *  When the <code>kind</code> is <code>CollectionEventKind.REPLACE</code>
     *  or <code>CollectionEventKind.UPDATE</code> the <code>items</code> property
     *  is an Array of PropertyChangeEvent objects with information about the items
     *  affected by the event.
     *  When a value changes, query the <code>newValue</code> and
     *  <code>oldValue</code> fields of the PropertyChangeEvent objects
     *  to find out what the old and new values were.
     *  When the <code>kind</code> is <code>CollectionEventKind.REFRESH</code>
     *  or <code>CollectionEventKind.RESET</code>, this array has zero length.
     *
     *  @default [ ]
     *
     *  @see PropertyChangeEvent
     * /
    public var items:Array;

	//----------------------------------
	//  location
	//----------------------------------

    /**
     *  When the <code>kind</code> value is <code>CollectionEventKind.ADD</code>,
     *  <code>CollectionEventKind.MOVE</code>,
     *  <code>CollectionEventKind.REMOVE</code>, or
     *  <code>CollectionEventKind.REPLACE</code>, this property is the 
	 *  zero-base index in the collection of the item(s) specified in the 
	 *  <code>items</code> property.
     *
     *  @see CollectionEventKind
     *
     *  @default -1
     * /
    public var location:int;

	//----------------------------------
	//  oldLocation
	//----------------------------------

    /**
     *  When the <code>kind</code> value is <code>CollectionEventKind.MOVE</code>,
     *  this property is the zero-based index in the target collection of the
     *  previous location of the item(s) specified by the <code>items</code> property.
     *
     *  @default -1
     * /
    public var oldLocation:int;

    //--------------------------------------------------------------------------
    //
    //  Overridden methods: Object
    //
    //--------------------------------------------------------------------------

    /**
     *  @private
     * /
    override public*/
    function toString() /*:String*/
    {
        return this.formatToString("CollectionEvent", "kind", "location", "oldLocation", "type", "bubbles", "cancelable", "eventPhase");
    }
    /*

    //--------------------------------------------------------------------------
    //
    //  Overridden methods: Event
    //
    //--------------------------------------------------------------------------

    /**
     *  @private
     * /
    override public*/
    function clone() /*:Event*/
    {
        return new CollectionEvent(this.type, this.bubbles, this.cancelable, this.kind, this.location, this.oldLocation, this.items);
    }
    /*
}

}

============================================== Jangaroo part ==============================================*/
    return {
        extend: "flash.events.Event",
        constructor: CollectionEvent$,
        kind: null,
        items: null,
        location: 0,
        oldLocation: 0,
        toString: toString,
        clone: clone,
        statics: {
            VERSION: "3.3.0.4852",
            COLLECTION_CHANGE: "collectionChange"
        },
        requires: [
            "flash.events.Event"
        ]
    };
});

Ext.define("mx.events.CollectionEventKind", function(CollectionEventKind) {
    /* ////////////////////////////////////////////////////////////////////////////////
//
//  ADOBE SYSTEMS INCORPORATED
//  Copyright 2005-2006 Adobe Systems Incorporated
//  All Rights Reserved.
//
//  NOTICE: Adobe permits you to use, modify, and distribute this file
//  in accordance with the terms of the license agreement accompanying it.
//
////////////////////////////////////////////////////////////////////////////////

package mx.events
{

/**
 *  The CollectionEventKind class contains constants for the valid values 
 *  of the mx.events.CollectionEvent class <code>kind</code> property.
 *  These constants indicate the kind of change that was made to the collection.
 *
 *  @see mx.events.CollectionEvent
 * /
public final class CollectionEventKind
{
	////////////////////////////////////////////////////////////////////////////////
//
//  ADOBE SYSTEMS INCORPORATED
//  Copyright 2005-2007 Adobe Systems Incorporated
//  All Rights Reserved.
//
//  NOTICE: Adobe permits you to use, modify, and distribute this file
//  in accordance with the terms of the license agreement accompanying it.
//
////////////////////////////////////////////////////////////////////////////////

import mx.core.mx_internal;

/**
 *  @private
 *  Version string for this class.
 * /
mx_internal static const VERSION:String = "3.3.0.4852";*/
    
    /*

    //--------------------------------------------------------------------------
    //
    //  Class constants
    //
    //--------------------------------------------------------------------------

    /**
     *  Indicates that the collection added an item or items.
     * /
    public static const ADD:String = "add";

    /**
     *  Indicates that the item has moved from the position identified
     *  by the CollectionEvent <code>oldLocation</code> property to the 
	 *  position identified by the <code>location</code> property.
     * /
    public static const MOVE:String = "move";

    /**
     *  Indicates that the collection applied a sort, a filter, or both.
     *  This change can potentially be easier to handle than a RESET.
     * /
    public static const REFRESH:String = "refresh";

    /**
     *  Indicates that the collection removed an item or items.
     * /
    public static const REMOVE:String = "remove";

    /**
     *  Indicates that the item at the position identified by the 
     *  CollectionEvent <code>location</code> property has been replaced.
     * /
    public static const REPLACE:String = "replace";

    /**
    *  Indicates that the collection has internally expanded. 
    *  This event kind occurs when a branch opens in a 
	*  hierarchical collection, for example when a Tree control branch opens.
    * /
    mx_internal static const EXPAND:String = "expand";

    /**
     *  Indicates that the collection has changed so drastically that
     *  a reset is required.
     * /
    public static const RESET:String = "reset";

    /**
     *  Indicates that one or more items were updated within the collection.
     *  The affected item(s) or their associated ObjectChangeEvent objects
     *  are stored in the <code>items</code> property.
     * /
    public static const UPDATE:String = "update";
}

}

============================================== Jangaroo part ==============================================*/
    return {
        statics: {
            VERSION: "3.3.0.4852",
            ADD: "add",
            MOVE: "move",
            REFRESH: "refresh",
            REMOVE: "remove",
            REPLACE: "replace",
            EXPAND: "expand",
            RESET: "reset",
            UPDATE: "update"
        }
    };
});

Ext.define("mx.utils.ArrayUtil", function(ArrayUtil) {
    /* ////////////////////////////////////////////////////////////////////////////////
//
//  ADOBE SYSTEMS INCORPORATED
//  Copyright 2004-2006 Adobe Systems Incorporated
//  All Rights Reserved.
//
//  NOTICE: Adobe permits you to use, modify, and distribute this file
//  in accordance with the terms of the license agreement accompanying it.
//
////////////////////////////////////////////////////////////////////////////////

package mx.utils
{

/**
 *  The ArrayUtil utility class is an all-static class
 *  with methods for working with arrays within Flex.
 *  You do not create instances of ArrayUtil;
 *  instead you call static methods such as the 
 *  <code>ArrayUtil.toArray()</code> method.
 * /
public class ArrayUtil
{
	////////////////////////////////////////////////////////////////////////////////
//
//  ADOBE SYSTEMS INCORPORATED
//  Copyright 2005-2007 Adobe Systems Incorporated
//  All Rights Reserved.
//
//  NOTICE: Adobe permits you to use, modify, and distribute this file
//  in accordance with the terms of the license agreement accompanying it.
//
////////////////////////////////////////////////////////////////////////////////

import mx.core.mx_internal;

/**
 *  @private
 *  Version string for this class.
 * /
mx_internal static const VERSION:String = "3.3.0.4852";*/
    
    /*

	//--------------------------------------------------------------------------
	//
	//  Class methods
	//
	//--------------------------------------------------------------------------

    /**
     *  Ensures that an Object can be used as an Array.
	 *
     *  <p>If the Object is already an Array, it returns the object. 
     *  If the object is not an Array, it returns an Array
	 *  in which the only element is the Object.
	 *  As a special case, if the Object is null,
	 *  it returns an empty Array.</p>
	 *
     *  @param obj Object that you want to ensure is an array.
	 *
     *  @return An Array. If the original Object is already an Array, 
     * 	the original Array is returned. Otherwise, a new Array whose
     *  only element is the Object is returned or an empty Array if 
     *  the Object was null. 
     * /
    public static*/
    function toArray$static(obj) /*:Object*/
    /*:Array*/
    {
        if (!obj)  {
            return [];
        }
        else if (AS3.is(obj, Array))  {
            return AS3.as(obj, Array);
        }
        else  {
            return [
                obj
            ];
        }
        
    }
    /*
    
    /**
     *  Returns the index of the item in the Array.
     * 
     *  Note that in this implementation the search is linear and is therefore 
     *  O(n).
     * 
     *  @param item The item to find in the Array. 
     *
     *  @param source The Array to search for the item.
     * 
     *  @return The index of the item, and -1 if the item is not in the list.
     * /
    public static*/
    function getItemIndex$static(item, /*:Object*/
    source) /*:Array*/
    /*:int*/
    {
        var n = /*:int*/
            source.length;
        for (var i = /*:int*/
            0; i < n; i++) {
            if (source[i] === item)  {
                return i;
            }
            
        }
        return -1;
    }
    /*
}

}

============================================== Jangaroo part ==============================================*/
    return {
        statics: {
            VERSION: "3.3.0.4852",
            toArray: toArray$static,
            getItemIndex: getItemIndex$static
        }
    };
});

Ext.define("mx.utils.DescribeTypeCache", function(DescribeTypeCache) {
    /*package mx.utils {
public class DescribeTypeCache {
  public*/
    function DescribeTypeCache$() {}
    /*

  static*/
    function describeType$static(obj) /*:Object*/
    /*:**/
    {
        return undefined;
    }
    /*

}
}

============================================== Jangaroo part ==============================================*/
    return {
        constructor: DescribeTypeCache$,
        statics: {
            describeType: describeType$static
        }
    };
});

Ext.define("mx.utils.ObjectUtil", function(ObjectUtil) {
    /* ////////////////////////////////////////////////////////////////////////////////
//
//  ADOBE SYSTEMS INCORPORATED
//  Copyright 2005-2007 Adobe Systems Incorporated
//  All Rights Reserved.
//
//  NOTICE: Adobe permits you to use, modify, and distribute this file
//  in accordance with the terms of the license agreement accompanying it.
//
////////////////////////////////////////////////////////////////////////////////

package mx.utils
{

//import flash.utils.ByteArray;
import flash.utils.Dictionary;
import flash.utils.getQualifiedClassName;
//import flash.xml.XMLNode;

import mx.collections.IList;

/**
 *  The ObjectUtil class is an all-static class with methods for
 *  working with Objects within Flex.
 *  You do not create instances of ObjectUtil;
 *  instead you simply call static methods such as the 
 *  <code>ObjectUtil.isSimple()</code> method.
 * /
public class ObjectUtil
{
    ////////////////////////////////////////////////////////////////////////////////
//
//  ADOBE SYSTEMS INCORPORATED
//  Copyright 2005-2007 Adobe Systems Incorporated
//  All Rights Reserved.
//
//  NOTICE: Adobe permits you to use, modify, and distribute this file
//  in accordance with the terms of the license agreement accompanying it.
//
////////////////////////////////////////////////////////////////////////////////

import mx.core.mx_internal;

/**
 *  @private
 *  Version string for this class.
 * /
mx_internal static const VERSION:String = "3.3.0.4852";*/
    
    /*
    
    /**
    *  Array of properties to exclude from debugging output.
    * /
    private static*/
    var defaultToStringExcludes$static;
    /*:Array*/
    /* =*/
    function defaultToStringExcludes$static_() {
        defaultToStringExcludes$static = ([
            "password",
            "credentials"
        ]);
    }
    
    /*

    //--------------------------------------------------------------------------
    //
    //  Class methods
    //
    //--------------------------------------------------------------------------

    /**
     *  Compares the Objects and returns an integer value 
     *  indicating if the first item is less than greater than or equal to
     *  the second item.
     *  This method will recursively compare properties on nested objects and
     *  will return as soon as a non-zero result is found.
     *  By default this method will recurse to the deepest level of any property.
     *  To change the depth for comparison specify a non-negative value for
     *  the <code>depth</code> parameter.
     *
     *  @param a Object.
     *
     *  @param b Object.
     *
     *  @param depth Indicates how many levels should be 
     *  recursed when performing the comparison.
     *  Set this value to 0 for a shallow comparison of only the primitive 
     *  representation of each property.
     *  For example:<pre>
     *  var a:Object = {name:"Bob", info:[1,2,3]};
     *  var b:Object = {name:"Alice", info:[5,6,7]};
     *  var c:int = ObjectUtil.compare(a, b, 0);</pre>
     *
     *  <p>In the above example the complex properties of <code>a</code> and 
     *  <code>b</code> will be flattened by a call to <code>toString()</code>
     *  when doing the comparison.
     *  In this case the <code>info</code> property will be turned into a string
     *  when performing the comparison.</p>
     *
     *  @return Return 0 if a and b are null, NaN, or equal. 
     *  Return 1 if a is null or greater than b. 
     *  Return -1 if b is null or greater than a. 
     * /
    public static*/
    function compare$static(a, /*:Object*/
    b, /*:Object*/
    depth) /*:int = -1*/
    /*:int*/
    {
        if (arguments.length <= 2)  {
            depth = -1;
        }
        
        return internalCompare$static(a, b, 0, depth, new flash.utils.Dictionary(true));
    }
    /*
    
    /**
     *  Copies the specified Object and returns a reference to the copy.
     *  The copy is made using a native serialization technique. 
     *  This means that custom serialization will be respected during the copy.
     *
     *  <p>This method is designed for copying data objects, 
     *  such as elements of a collection. It is not intended for copying 
     *  a UIComponent object, such as a TextInput control. If you want to create copies 
     *  of specific UIComponent objects, you can create a subclass of the component and implement 
     *  a <code>clone()</code> method, or other method to perform the copy.</p>
     * 
     *  @param value Object that should be copied.
     * 
     *  @return Copy of the specified Object.
     * / 
    public static*/
    function copy$static(value) /*:Object*/
    /*:Object*/
    {
        //        var buffer:ByteArray = new ByteArray();
        //        buffer.writeObject(value);
        //        buffer.position = 0;
        //        var result:Object = buffer.readObject();
        //        return result;
        return null;
    }
    /*
    
    /**
     *  Returns <code>true</code> if the object reference specified
     *  is a simple data type. The simple data types include the following:
     *  <ul>
     *    <li><code>String</code></li>
     *    <li><code>Number</code></li>
     *    <li><code>uint</code></li>
     *    <li><code>int</code></li>
     *    <li><code>Boolean</code></li>
     *    <li><code>Date</code></li>
     *    <li><code>Array</code></li>
     *  </ul>
     *
     *  @param value Object inspected.
     *
     *  @return <code>true</code> if the object specified
     *  is one of the types above; <code>false</code> otherwise.
     * /
    public static*/
    function isSimple$static(value) /*:Object*/
    /*:Boolean*/
    {
        var type = /*:String*/
            typeof (value);
        switch (type) {
            case "number":
            case "string":
            case "boolean":
                {
                    return true;
                };
            case "object":
                {
                    return (AS3.is(value, Date)) || (AS3.is(value, Array));
                };
        }
        return false;
    }
    /*

    /**
     *  Compares two numeric values.
     * 
     *  @param a First number.
     * 
     *  @param b Second number.
     *
     *  @return 0 is both numbers are NaN. 
     *  1 if only <code>a</code> is a NaN.
     *  -1 if only <code>b</code> is a NaN.
     *  -1 if <code>a</code> is less than <code>b</code>.
     *  1 if <code>a</code> is greater than <code>b</code>.
     * /
    public static*/
    function numericCompare$static(a, /*:Number*/
    b) /*:Number*/
    /*:int*/
    {
        if (isNaN(a) && isNaN(b))  {
            return 0;
        }
        
        if (isNaN(a))  {
            return 1;
        }
        
        if (isNaN(b))  {
            return -1;
        }
        
        if (a < b)  {
            return -1;
        }
        
        if (a > b)  {
            return 1;
        }
        
        return 0;
    }
    /*

    /**
     *  Compares two String values.
     * 
     *  @param a First String value.
     * 
     *  @param b Second String value.
     *
     *  @param caseInsensitive Specifies to perform a case insensitive compare, 
     *  <code>true</code>, or not, <code>false</code>.
     *
     *  @return 0 is both Strings are null. 
     *  1 if only <code>a</code> is null.
     *  -1 if only <code>b</code> is null.
     *  -1 if <code>a</code> precedes <code>b</code>.
     *  1 if <code>b</code> precedes <code>a</code>.
     * /
    public static*/
    function stringCompare$static(a, /*:String*/
    b, /*:String*/
    caseInsensitive) /*:Boolean = false*/
    /*:int*/
    {
        if (arguments.length <= 2)  {
            caseInsensitive = false;
        }
        
        if (a == null && b == null)  {
            return 0;
        }
        
        if (a == null)  {
            return 1;
        }
        
        if (b == null)  {
            return -1;
        }
        
        // Convert to lowercase if we are case insensitive.
        if (caseInsensitive) {
            a = a.toLocaleLowerCase();
            b = b.toLocaleLowerCase();
        }
        var result = /*:int*/
            a.localeCompare(b);
        if (result < -1)  {
            result = -1;
        }
        else if (result > 1)  {
            result = 1;
        }
        
        return result;
    }
    /*

    /**
     *  Compares the two Date objects and returns an integer value 
     *  indicating if the first Date object is before, equal to, 
     *  or after the second item.
     *
     *  @param a Date object.
     *
     *  @param b Date object.
     *
     *  @return 0 if <code>a</code> and <code>b</code>
     *  are <code>null</code> or equal; 
     *  1 if <code>a</code> is <code>null</code>
     *  or before <code>b</code>; 
     *  -1 if <code>b</code> is <code>null</code>
     *  or before <code>a</code>. 
     * /
    public static*/
    function dateCompare$static(a, /*:Date*/
    b) /*:Date*/
    /*:int*/
    {
        if (a == null && b == null)  {
            return 0;
        }
        
        if (a == null)  {
            return 1;
        }
        
        if (b == null)  {
            return -1;
        }
        
        var na = /*:Number*/
            a.getTime();
        var nb = /*:Number*/
            b.getTime();
        if (na < nb)  {
            return -1;
        }
        
        if (na > nb)  {
            return 1;
        }
        
        return 0;
    }
    /*
        
    /**
     *  Pretty-prints the specified Object into a String.
     *  All properties will be in alpha ordering.
     *  Each object will be assigned an id during printing;
     *  this value will be displayed next to the object type token
     *  preceded by a '#', for example:
     *
     *  <pre>
     *  (mx.messaging.messages::AsyncMessage)#2.</pre>
     *
     *  <p>This id is used to indicate when a circular reference occurs.
     *  Properties of an object that are of the <code>Class</code> type will
     *  appear only as the assigned type.
     *  For example a custom definition like the following:</p>
     *
     *  <pre>
     *    public class MyCustomClass {
     *      public var clazz:Class;
     *    }</pre>
     * 
     *  <p>With the <code>clazz</code> property assigned to <code>Date</code>
     *  will display as shown below:</p>
     * 
     *  <pre>
     *   (somepackage::MyCustomClass)#0
     *      clazz = (Date)</pre>
     *
     *  @param obj Object to be pretty printed.
     * 
     *  @param namespaceURIs Array of namespace URIs for properties 
     *  that should be included in the output.
     *  By default only properties in the public namespace will be included in
     *  the output.
     *  To get all properties regardless of namespace pass an array with a 
     *  single element of "*".
     * 
     *  @param exclude Array of the property names that should be 
     *  excluded from the output.
     *  Use this to remove data from the formatted string.
     * 
     *  @return String containing the formatted version
     *  of the specified object.
     *
     *  @example
     *  <pre>
     *  // example 1
     *  var obj:AsyncMessage = new AsyncMessage();
     *  obj.body = [];
     *  obj.body.push(new AsyncMessage());
     *  obj.headers["1"] = { name: "myName", num: 15.3};
     *  obj.headers["2"] = { name: "myName", num: 15.3};
     *  obj.headers["10"] = { name: "myName", num: 15.3};
     *  obj.headers["11"] = { name: "myName", num: 15.3};
     *  trace(ObjectUtil.toString(obj));
     *
     *  // will output to flashlog.txt
     *  (mx.messaging.messages::AsyncMessage)#0
     *    body = (Array)#1
     *      [0] (mx.messaging.messages::AsyncMessage)#2
     *        body = (Object)#3
     *        clientId = (Null)
     *        correlationId = ""
     *        destination = ""
     *        headers = (Object)#4
     *        messageId = "378CE96A-68DB-BC1B-BCF7FFFFFFFFB525"
     *        sequenceId = (Null)
     *        sequencePosition = 0
     *        sequenceSize = 0
     *        timeToLive = 0
     *        timestamp = 0
     *    clientId = (Null)
     *    correlationId = ""
     *    destination = ""
     *    headers = (Object)#5
     *      1 = (Object)#6
     *        name = "myName"
     *        num = 15.3
     *      10 = (Object)#7
     *        name = "myName"
     *        num = 15.3
     *      11 = (Object)#8
     *        name = "myName"
     *        num = 15.3
     *      2 = (Object)#9
     *        name = "myName"
     *        num = 15.3
     *    messageId = "1D3E6E96-AC2D-BD11-6A39FFFFFFFF517E"
     *    sequenceId = (Null)
     *    sequencePosition = 0
     *    sequenceSize = 0
     *    timeToLive = 0
     *    timestamp = 0
     *
     *  // example 2 with circular references
     *  obj = {};
     *  obj.prop1 = new Date();
     *  obj.prop2 = [];
     *  obj.prop2.push(15.2);
     *  obj.prop2.push("testing");
     *  obj.prop2.push(true);
     *  obj.prop3 = {};
     *  obj.prop3.circular = obj;
     *  obj.prop3.deeper = new ErrorMessage();
     *  obj.prop3.deeper.rootCause = obj.prop3.deeper;
     *  obj.prop3.deeper2 = {};
     *  obj.prop3.deeper2.deeperStill = {};
     *  obj.prop3.deeper2.deeperStill.yetDeeper = obj;
     *  trace(ObjectUtil.toString(obj));
     *
     *  // will output to flashlog.txt
     *  (Object)#0
     *    prop1 = Tue Apr 26 13:59:17 GMT-0700 2005
     *    prop2 = (Array)#1
     *      [0] 15.2
     *      [1] "testing"
     *      [2] true
     *    prop3 = (Object)#2
     *      circular = (Object)#0
     *      deeper = (mx.messaging.messages::ErrorMessage)#3
     *        body = (Object)#4
     *        clientId = (Null)
     *        code = (Null)
     *        correlationId = ""
     *        destination = ""
     *        details = (Null)
     *        headers = (Object)#5
     *        level = (Null)
     *        message = (Null)
     *        messageId = "14039376-2BBA-0D0E-22A3FFFFFFFF140A"
     *        rootCause = (mx.messaging.messages::ErrorMessage)#3
     *        sequenceId = (Null)
     *        sequencePosition = 0
     *        sequenceSize = 0
     *        timeToLive = 0
     *        timestamp = 0
     *      deeper2 = (Object)#6
     *        deeperStill = (Object)#7
     *          yetDeeper = (Object)#0
     * 
     * // example 3 with Dictionary
     * var point:Point = new Point(100, 100);
     * var point2:Point = new Point(100, 100);
     * var obj:Dictionary = new Dictionary();
     * obj[point] = "point";
     * obj[point2] = "point2";
     * obj["1"] = { name: "one", num: 1};
     * obj["two"] = { name: "2", num: 2};
     * obj[3] = 3;
     * trace(ObjectUtil.toString(obj));
     * 
     * // will output to flashlog.txt
     * (flash.utils::Dictionary)#0
     *   {(flash.geom::Point)#1
     *     length = 141.4213562373095
     *     x = 100
     *     y = 100} = "point2"
     *   {(flash.geom::Point)#2
     *     length = 141.4213562373095
     *     x = 100
     *     y = 100} = "point"
     *   {1} = (Object)#3
     *     name = "one"
     *     num = 1
     *   {3} = 3
     *   {"two"} = (Object)#4
     *     name = "2"
     *     num = 2
     * 
     * </pre>
     * /
    public static*/
    function toString$static(value, /*:Object*/
    namespaceURIs, /*:Array = null*/
    exclude) /*:Array = null*/
    /*:String*/
    {
        switch (Math.max(arguments.length, 1)) {
            case 1:
                namespaceURIs = null;
            case 2:
                exclude = null;
        }
        if (exclude == null) {
            exclude = defaultToStringExcludes$static;
        }
        refCount$static = 0;
        return internalToString$static(value, 0, null, namespaceURIs, exclude);
    }
    /*
    
    /**
     *  This method cleans up all of the additional parameters that show up in AsDoc
     *  code hinting tools that developers shouldn't ever see.
     *  @private
     * /
    private static*/
    function internalToString$static(value, /*:Object*/
    indent, /*:int = 0*/
    refs, /*:Dictionary= null*/
    namespaceURIs, /*:Array = null*/
    exclude) /*:Array = null*/
    /*:String*/
    {
        switch (Math.max(arguments.length, 1)) {
            case 1:
                indent = 0;
            case 2:
                refs = null;
            case 3:
                namespaceURIs = null;
            case 4:
                exclude = null;
        }
        var str;
        /*:String*/
        var type = /*:String*/
            value == null ? "null" : typeof (value);
        switch (type) {
            case "boolean":
            case "number":
                {
                    return value.toString();
                };
            case "string":
                {
                    return "\"" + value.toString() + "\"";
                };
            case "object":
                {
                    if (AS3.is(value, Date)) {
                        return value.toString();
                    }
                    //                else if (value is XMLNode)
                    //                {
                    //                    return value.toString();
                    //                }
                    else if (AS3.is(value, Class)) {
                        return "(" + flash.utils.getQualifiedClassName(value) + ")";
                    } else {
                        var classInfo = /*:Object*/
                            ObjectUtil.getClassInfo(value, exclude, {
                                includeReadOnly: true,
                                uris: namespaceURIs
                            });
                        var properties = /*:Array*/
                            classInfo.properties;
                        str = "(" + classInfo.name + ")";
                        // refs help us avoid circular reference infinite recursion.
                        // Each time an object is encoumtered it is pushed onto the
                        // refs stack so that we can determine if we have visited
                        // this object already.
                        if (refs == null)  {
                            refs = new flash.utils.Dictionary(true);
                        }
                        
                        // Check to be sure we haven't processed this object before
                        var id = /*:Object*/
                            refs[value];
                        if (id != null) {
                            str += "#" + id;
                            return str;
                        }
                        if (value != null) {
                            str += "#" + refCount$static;
                            refs[value] = refCount$static;
                            refCount$static++;
                        }
                        var isArray = /*:Boolean*/
                            AS3.is(value, Array);
                        var isDict = /*:Boolean*/
                            AS3.is(value, flash.utils.Dictionary);
                        var prop;
                        /*:**/
                        indent += 2;
                        // Print all of the variable values.
                        for (var j = /*:int*/
                            0; j < properties.length; j++) {
                            str = newline$static(str, indent);
                            prop = properties[j];
                            if (isArray)  {
                                str += "[";
                            }
                            else if (isDict)  {
                                str += "{";
                            }
                            
                            if (isDict) {
                                // in dictionaries, recurse on the key, because it can be a complex object
                                str += internalToString$static(prop, indent, refs, namespaceURIs, exclude);
                            } else {
                                str += prop.toString();
                            }
                            if (isArray)  {
                                str += "] ";
                            }
                            else if (isDict)  {
                                str += "} = ";
                            }
                            else  {
                                str += " = ";
                            }
                            
                            try {
                                // print the value
                                str += internalToString$static(value[prop], indent, refs, namespaceURIs, exclude);
                            } catch (e) {
                                if (AS3.is(e, AS3.Error)) {
                                    // value[prop] can cause an RTE
                                    // for certain properties of certain objects.
                                    // For example, accessing the properties
                                    //   actionScriptVersion
                                    //   childAllowsParent
                                    //   frameRate
                                    //   height
                                    //   loader
                                    //   parentAllowsChild
                                    //   sameDomain
                                    //   swfVersion
                                    //   width
                                    // of a Stage's loaderInfo causes
                                    //   Error #2099: The loading object is not
                                    //   sufficiently loaded to provide this information
                                    // In this case, we simply output ? for the value.
                                    str += "?";
                                } else  {
                                    throw e;
                                }
                                
                            }
                        }
                        indent -= 2;
                        return str;
                    }
                    break;
                };
            case "xml":
                {
                    return value.toString();
                };
            default:
                {
                    return "(" + type + ")";
                };
        }
        return "(unknown)";
    }
    /*

    /**
     *  @private
     *  This method will append a newline and the specified number of spaces
     *  to the given string.
     * /
    private static*/
    function newline$static(str, /*:String*/
    n) /*:int = 0*/
    /*:String*/
    {
        if (arguments.length <= 1)  {
            n = 0;
        }
        
        var result = /*:String*/
            str;
        result += "\n";
        for (var i = /*:int*/
            0; i < n; i++) {
            result += " ";
        }
        return result;
    }
    /*
    
    private static*/
    function internalCompare$static(a, /*:Object*/
    b, /*:Object*/
    currentDepth, /*:int*/
    desiredDepth, /*:int*/
    refs) /*:Dictionary*/
    /*:int*/
    {
        if (a == null && b == null)  {
            return 0;
        }
        
        if (a == null)  {
            return 1;
        }
        
        if (b == null)  {
            return -1;
        }
        
        //        if (a is ObjectProxy)
        //            a = ObjectProxy(a).object_proxy::object;
        //            
        //        if (b is ObjectProxy)
        //            b = ObjectProxy(b).object_proxy::object;
        var typeOfA = /*:String*/
            typeof (a);
        var typeOfB = /*:String*/
            typeof (b);
        var result = /*:int*/
            0;
        if (typeOfA == typeOfB) {
            switch (typeOfA) {
                case "boolean":
                    {
                        result = ObjectUtil.numericCompare(Number(a), Number(b));
                        break;
                    };
                case "number":
                    {
                        result = ObjectUtil.numericCompare(AS3.as(a, Number), AS3.as(b, Number));
                        break;
                    };
                case "string":
                    {
                        result = ObjectUtil.stringCompare(AS3.as(a, String), AS3.as(b, String));
                        break;
                    };
                case "object":
                    {
                        var newDepth = /*:int*/
                            desiredDepth > 0 ? desiredDepth - 1 : desiredDepth;
                        // refs help us avoid circular reference infinite recursion.
                        // Each time an object is encoumtered it is pushed onto the
                        // refs stack so that we can determine if we have visited
                        // this object already.
                        // Here since we are comparing two objects we can short
                        // circuit at the first encounter but have to exhaust all
                        // references found.  A visited reference makes an object 
                        // "greater" than another object, only if both objects
                        // have a visited reference will the result be 0
                        var aRef = /*:Boolean*/
                            refs[a];
                        var bRef = /*:Boolean*/
                            refs[b];
                        if (aRef && !bRef)  {
                            return 1;
                        }
                        else if (bRef && !aRef)  {
                            return -1;
                        }
                        else if (bRef && aRef)  {
                            return 0;
                        }
                        
                        refs[a] = true;
                        refs[b] = true;
                        if (desiredDepth != -1 && (currentDepth > desiredDepth)) {
                            // once we try to go beyond the desired depth we should 
                            // toString() our way out
                            result = ObjectUtil.stringCompare(a.toString(), b.toString());
                        } else if ((AS3.is(a, Array)) && (AS3.is(b, Array))) {
                            result = arrayCompare$static(AS3.as(a, Array), AS3.as(b, Array), currentDepth, desiredDepth, refs);
                        } else if ((AS3.is(a, Date)) && (AS3.is(b, Date))) {
                            result = ObjectUtil.dateCompare(AS3.as(a, Date), AS3.as(b, Date));
                        } else if ((AS3.is(a, mx.collections.IList)) && (AS3.is(b, mx.collections.IList))) {
                            result = listCompare$static(AS3.as(a, mx.collections.IList), AS3.as(b, mx.collections.IList), currentDepth, desiredDepth, refs);
                        }
                        //                    else if ((a is ByteArray) && (b is ByteArray))
                        //                    {
                        //                        result = byteArrayCompare(a as ByteArray, b as ByteArray);
                        //                    }
                        else if (flash.utils.getQualifiedClassName(a) == flash.utils.getQualifiedClassName(b)) {
                            var aProps = /*:Array*/
                                ObjectUtil.getClassInfo(a).properties;
                            var bProps;
                            /*:Array*/
                            // if the objects are anonymous they could have different 
                            // # of properties and should be treated on that basis first
                            if (flash.utils.getQualifiedClassName(a) == "Object") {
                                bProps = ObjectUtil.getClassInfo(b).properties;
                                result = arrayCompare$static(aProps, bProps, currentDepth, newDepth, refs);
                            }
                            if (result != 0) {
                                return result;
                            }
                            // loop through all of the properties and compare
                            var propName;
                            /*:QName*/
                            var aProp;
                            /*:Object*/
                            var bProp;
                            /*:Object*/
                            for (var i = /*:int*/
                                0; i < aProps.length; i++) {
                                propName = aProps[i];
                                aProp = a[propName];
                                bProp = b[propName];
                                result = internalCompare$static(aProp, bProp, currentDepth + 1, newDepth, refs);
                                if (result != 0) {
                                    i = aProps.length;
                                }
                            }
                        } else {
                            // We must be inequal, so return 1
                            return 1;
                        }
                        break;
                    };
            }
        } else // be consistent with the order we return here
        {
            return ObjectUtil.stringCompare(typeOfA, typeOfB);
        }
        return result;
    }
    /*
    
    /**
     *  Returns information about the class, and properties of the class, for
     *  the specified Object.
     *
     *  @param obj The Object to inspect.
     *
     *  @param exclude Array of Strings specifying the property names that should be 
     *  excluded from the returned result. For example, you could specify 
     *  <code>["currentTarget", "target"]</code> for an Event object since these properties 
     *  can cause the returned result to become large.
     *
     *  @param options An Object containing one or more properties 
     *  that control the information returned by this method. 
     *  The properties include the following:
     *
     *  <ul>
     *    <li><code>includeReadOnly</code>: If <code>false</code>, 
     *      exclude Object properties that are read-only. 
     *      The default value is <code>true</code>.</li>
     *  <li><code>includeTransient</code>: If <code>false</code>, 
     *      exclude Object properties and variables that have <code>[Transient]</code> metadata.
     *      The default value is <code>true</code>.</li>
     *  <li><code>uris</code>: Array of Strings of all namespaces that should be included in the output.
     *      It does allow for a wildcard of "~~". 
     *      By default, it is null, meaning no namespaces should be included. 
     *      For example, you could specify <code>["mx_internal", "mx_object"]</code> 
     *      or <code>["~~"]</code>.</li>
     *  </ul>
     * 
     *  @return An Object containing the following properties:
     *  <ul>
     *    <li><code>name</code>: String containing the name of the class;</li>
     *    <li><code>properties</code>: Sorted list of the property names of the specified object,
     *    or references to the original key if the specified object is a Dictionary</li>
     *  </ul>
    * /
    public static*/
    function getClassInfo$static(obj, /*:Object*/
    excludes, /*:Array = null*/
    options) /*:Object = null*/
    /*:Object*/
    {
        switch (Math.max(arguments.length, 1)) {
            case 1:
                excludes = null;
            case 2:
                options = null;
        }
        //        var n:int;
        //        var i:int;
        //
        ////        if (obj is ObjectProxy)
        ////            obj = ObjectProxy(obj).object_proxy::object;
        //
        //        if (options == null)
        //            options = { includeReadOnly: true, uris: null, includeTransient: true };
        //
        //        var result:Object;
        //        var propertyNames:Array = [];
        //        var cacheKey:String;
        //
        //        var className:String;
        //        var classAlias:String;
        //        var properties:XMLList;
        //        var prop:XML;
        //        var dynamic:Boolean = false;
        //        var metadataInfo:Object;
        //
        //        if (typeof(obj) == "xml")
        //        {
        //            className = "XML";
        //            properties = obj.text();
        //            if (properties.length())
        //                propertyNames.push("*");
        //            properties = obj.attributes();
        //        }
        //        else
        //        {
        //            var classInfo:XML = DescribeTypeCache.describeType(obj).typeDescription;
        //            className = classInfo["@name"].toString();
        //            classAlias = classInfo["@alias"].toString();
        //            dynamic = (classInfo["@isDynamic"].toString() == "true");
        //
        ////            if (options.includeReadOnly)
        ////                properties = classInfo..accessor.(@access != "writeonly") + classInfo..variable;
        ////            else
        ////                properties = classInfo..accessor.(@access == "readwrite") + classInfo..variable;
        //
        //            var numericIndex:Boolean = false;
        //        }
        //
        //        // If type is not dynamic, check our cache for class info...
        //        if (!dynamic)
        //        {
        //            cacheKey = getCacheKey(obj, excludes, options);
        //            result = CLASS_INFO_CACHE[cacheKey];
        //            if (result != null)
        //                return result;
        //        }
        //
        //        result = {};
        //        result["name"] = className;
        //        result["alias"] = classAlias;
        //        result["properties"] = propertyNames;
        //        result["dynamic"] = dynamic;
        //        result["metadata"] = metadataInfo = recordMetadata(properties);
        //
        //        var excludeObject:Object = {};
        //        if (excludes)
        //        {
        //            n = excludes.length;
        //            for (i = 0; i < n; i++)
        //            {
        //                excludeObject[excludes[i]] = 1;
        //            }
        //        }
        //
        //        //TODO this seems slightly fragile, why not use the 'is' operator?
        //        var isArray:Boolean = (className == "Array");
        //        var isDict:Boolean  = (className == "flash.utils::Dictionary");
        //
        //        if (isDict)
        //        {
        //            // dictionaries can have multiple keys of the same type,
        //            // (they can index by reference rather than QName, String, or number),
        //            // which cannot be looked up by QName, so use references to the actual key
        //            for (var key:* in obj)
        //            {
        //                propertyNames.push(key);
        //            }
        //        }
        //        else if (dynamic)
        //        {
        //            for (var p:String in obj)
        //            {
        //                if (excludeObject[p] != 1)
        //                {
        //                    if (isArray)
        //                    {
        //                         var pi:Number = parseInt(p);
        //                         if (isNaN(pi))
        //                            propertyNames.push(new QName("", p));
        //                         else
        //                            propertyNames.push(pi);
        //                    }
        //                    else
        //                    {
        //                        propertyNames.push(new QName("", p));
        //                    }
        //                }
        //            }
        //            numericIndex = isArray && !isNaN(Number(p));
        //        }
        //
        //        if (isArray || isDict || className == "Object")
        //        {
        //            // Do nothing since we've already got the dynamic members
        //        }
        //        else if (className == "XML")
        //        {
        //            n = properties.length();
        //            for (i = 0; i < n; i++)
        //            {
        //                p = properties[i].name();
        //                if (excludeObject[p] != 1)
        //                    propertyNames.push(new QName("", "@" + p));
        //            }
        //        }
        //        else
        //        {
        //            n = properties.length();
        //            var uris:Array = options.uris;
        //            var uri:String;
        //            var qName:QName;
        //            for (i = 0; i < n; i++)
        //            {
        //                prop = properties[i];
        //                p = prop["@name"].toString();
        //                uri = prop["@uri"].toString();
        //
        //                if (excludeObject[p] == 1)
        //                    continue;
        //
        //                if (!options.includeTransient && internalHasMetadata(metadataInfo, p, "Transient"))
        //                    continue;
        //
        //                if (uris != null)
        //                {
        //                    if (uris.length == 1 && uris[0] == "*")
        //                    {
        //                        qName = new QName(uri, p);
        //                        try
        //                        {
        //                            obj[qName]; // access the property to ensure it is supported
        //                            propertyNames.push();
        //                        }
        //                        catch(e:Error)
        //                        {
        //                            // don't keep property name
        //                        }
        //                    }
        //                    else
        //                    {
        //                        for (var j:int = 0; j < uris.length; j++)
        //                        {
        //                            uri = uris[j];
        //                            if (prop["@uri"].toString() == uri)
        //                            {
        //                                qName = new QName(uri, p);
        //                                try
        //                                {
        //                                    obj[qName];
        //                                    propertyNames.push(qName);
        //                                }
        //                                catch(e:Error)
        //                                {
        //                                    // don't keep property name
        //                                }
        //                            }
        //                        }
        //                    }
        //                }
        //                else if (uri.length == 0)
        //                {
        //                    qName = new QName(uri, p);
        //                    try
        //                    {
        //                        obj[qName];
        //                        propertyNames.push(qName);
        //                    }
        //                    catch(e:Error)
        //                    {
        //                        // don't keep property name
        //                    }
        //                }
        //            }
        //        }
        //
        //        propertyNames.sort(Array.CASEINSENSITIVE |
        //                           (numericIndex ? Array.NUMERIC : 0));
        //
        //        // dictionary keys can be indexed by an object reference
        //        // there's a possibility that two keys will have the same toString()
        //        // so we don't want to remove dupes
        //        if (!isDict)
        //        {
        //            // for Arrays, etc., on the other hand...
        //            // remove any duplicates, i.e. any items that can't be distingushed by toString()
        //            for (i = 0; i < propertyNames.length - 1; i++)
        //            {
        //                // the list is sorted so any duplicates should be adjacent
        //                // two properties are only equal if both the uri and local name are identical
        //                if (propertyNames[i].toString() == propertyNames[i + 1].toString())
        //                {
        //                    propertyNames.splice(i, 1);
        //                    i--; // back up
        //                }
        //            }
        //        }
        //
        //        // For normal, non-dynamic classes we cache the class info
        //        if (!dynamic)
        //        {
        //            cacheKey = getCacheKey(obj, excludes, options);
        //            CLASS_INFO_CACHE[cacheKey] = result;
        //        }
        //
        //        return result;
        return null;
    }
    /*

    /**
     * Uses <code>getClassInfo</code> and examines the metadata information to
     * determine whether a property on a given object has the specified 
     * metadata.
     * 
     * @param obj The object holding the property.
     * @param propName The property to check for metadata.
     * @param metadataName The name of the metadata to check on the property.
     * @param excludes If any properties need to be excluded when generating class info. (Optional)
     * @param options If any options flags need to changed when generating class info. (Optional)
     * @return true if the property has the specified metadata.
     * /
    public static*/
    function hasMetadata$static(obj, /*:Object*/
    propName, /*:String*/
    metadataName, /*:String*/
    excludes, /*:Array = null*/
    options) /*:Object = null*/
    /*:Boolean*/
    {
        switch (Math.max(arguments.length, 3)) {
            case 3:
                excludes = null;
            case 4:
                options = null;
        }
        var classInfo = /*:Object*/
            ObjectUtil.getClassInfo(obj, excludes, options);
        var metadataInfo = /*:Object*/
            classInfo["metadata"];
        return internalHasMetadata$static(metadataInfo, propName, metadataName);
    }
    /*

    /**
     *  @private
     * /
    private static*/
    function internalHasMetadata$static(metadataInfo, /*:Object*/
    propName, /*:String*/
    metadataName) /*:String*/
    /*:Boolean*/
    {
        if (metadataInfo != null) {
            var metadata = /*:Object*/
                metadataInfo[propName];
            if (metadata != null) {
                if (metadata[metadataName] != null)  {
                    return true;
                }
                
            }
        }
        return false;
    }
    /*

    /**
     *  @private
     * /
//    private static function recordMetadata(properties:XMLList):Object
//    {
//        var result:Object = null;
//      /*
//
//        try
//        {
//            for each (var prop:XML in properties)
//            {
//                var propName:String = prop.attribute("name").toString();
//                var metadataList:XMLList = prop.metadata;
//
//                if (metadataList.length() > 0)
//                {
//                    if (result == null)
//                        result = {};
//
//                    var metadata:Object = {};
//                    result[propName] = metadata;
//
//                    for each (var md:XML in metadataList)
//                    {
//                        var mdName:String = md.attribute("name").toString();
//
//                        var argsList:XMLList = md.arg;
//                        var value:Object = {};
//
//                        for each (var arg:XML in argsList)
//                        {
//                            var argKey:String = arg.attribute("key").toString();
//                            if (argKey != null)
//                            {
//                                var argValue:String = arg.attribute("value").toString();
//                                value[argKey] = argValue;
//                            }
//                        }
//
//                        var existing:Object = metadata[mdName];
//                        if (existing != null)
//                        {
//                            var existingArray:Array;
//                            if (existing is Array)
//                                existingArray = existing as Array;
//                            else
//                                existingArray = [];
//                            existingArray.push(value);
//                            existing = existingArray;
//                        }
//                        else
//                        {
//                            existing = value;
//                        }
//                        metadata[mdName] = existing;
//                    }
//                }
//            }
//        }
//        catch(e:Error)
//        {
//        }
//    * /
//
//        return result;
//    }

    /**
     *  @private
     * /
    private static*/
    function getCacheKey$static(o, /*:Object*/
    excludes, /*:Array = null*/
    options) /*:Object = null*/
    /*:String*/
    {
        switch (Math.max(arguments.length, 1)) {
            case 1:
                excludes = null;
            case 2:
                options = null;
        }
        var key = /*:String*/
            flash.utils.getQualifiedClassName(o);
        if (excludes != null) {
            for (var i = /*:uint*/
                0; i < excludes.length; i++) {
                var excl = /*:String*/
                    AS3.as(excludes[i], String);
                if (excl != null)  {
                    key += excl;
                }
                
            }
        }
        if (options != null) {
            for (var flag in /*:String*/
            options) {
                key += flag;
                var value = /*:String*/
                    AS3.as(options[flag], String);
                if (value != null)  {
                    key += value;
                }
                
            }
        }
        return key;
    }
    /*

    /**
     *  @private
     * /
    private static*/
    function arrayCompare$static(a, /*:Array*/
    b, /*:Array*/
    currentDepth, /*:int*/
    desiredDepth, /*:int*/
    refs) /*:Dictionary*/
    /*:int*/
    {
        var result = /*:int*/
            0;
        if (a.length != b.length) {
            if (a.length < b.length)  {
                result = -1;
            }
            else  {
                result = 1;
            }
            
        } else {
            var key;
            /*:String*/
            for (var $1 = 0; $1 < /* in*/
            a.length; ++$1) {
                key = String($1);
                if (b.hasOwnProperty(key)) {
                    result = internalCompare$static(a[key], b[key], currentDepth, desiredDepth, refs);
                    if (result != 0)  {
                        return result;
                    }
                    
                } else {
                    return -1;
                }
            }
            for (var $2 = 0; $2 < /* in*/
            b.length; ++$2) {
                key = String($2);
                if (!a.hasOwnProperty(key)) {
                    return 1;
                }
            }
        }
        return result;
    }
    /*
    
    /**
     * @private
     * /
//    private static function byteArrayCompare(a:ByteArray, b:ByteArray):int
//    {
//        var result:int = 0;
//        if (a.length != b.length)
//        {
//            if (a.length < b.length)
//                result = -1;
//            else
//                result = 1;
//        }
//        else
//        {
//            a.position = 0;
//            b.position = 0;
//            for (var i:int = 0; i < a.length; i++)
//            {
//                result = numericCompare(a.readByte(), b.readByte());
//                if (result != 0)
//                {
//                    i = a.length;
//                }
//            }
//        }
//        return result;
//    }

    /**
     *  @private
     * /
    private static*/
    function listCompare$static(a, /*:IList*/
    b, /*:IList*/
    currentDepth, /*:int*/
    desiredDepth, /*:int*/
    refs) /*:Dictionary*/
    /*:int*/
    {
        var result = /*:int*/
            0;
        if (a.length != b.length) {
            if (a.length < b.length)  {
                result = -1;
            }
            else  {
                result = 1;
            }
            
        } else {
            for (var i = /*:int*/
                0; i < a.length; i++) {
                result = internalCompare$static(a.getItemAt(i), b.getItemAt(i), currentDepth + 1, desiredDepth, refs);
                if (result != 0) {
                    i = a.length;
                }
            }
        }
        return result;
    }
    /*
    
    /**
     * @private
     * /
    private static*/
    var refCount$static = /*:int*/
        0;
    /*

    /**
     * @private
     * / 
    private static*/
    var CLASS_INFO_CACHE$static;
    /*:Object*/
    /* =*/
    function CLASS_INFO_CACHE$static_() {
        CLASS_INFO_CACHE$static = ({});
    }
    
    /*
}

}

============================================== Jangaroo part ==============================================*/
    return {
        statics: {
            VERSION: "3.3.0.4852",
            defaultToStringExcludes: undefined,
            compare: compare$static,
            copy: copy$static,
            isSimple: isSimple$static,
            numericCompare: numericCompare$static,
            stringCompare: stringCompare$static,
            dateCompare: dateCompare$static,
            toString: toString$static,
            getClassInfo: getClassInfo$static,
            hasMetadata: hasMetadata$static,
            CLASS_INFO_CACHE: undefined,
            __initStatics__: function() {
                defaultToStringExcludes$static_();
                CLASS_INFO_CACHE$static_();
            }
        },
        requires: [
            "AS3.Error",
            "flash.utils.Dictionary",
            "flash.utils.getQualifiedClassName"
        ],
        uses: [
            "mx.collections.IList"
        ]
    };
});

Ext.define("mx.utils.StringUtil", function(StringUtil) {
    /* ////////////////////////////////////////////////////////////////////////////////
//
//  ADOBE SYSTEMS INCORPORATED
//  Copyright 2005-2007 Adobe Systems Incorporated
//  All Rights Reserved.
//
//  NOTICE: Adobe permits you to use, modify, and distribute this file
//  in accordance with the terms of the license agreement accompanying it.
//
////////////////////////////////////////////////////////////////////////////////

package mx.utils
{

/**
 *  The StringUtil utility class is an all-static class with methods for
 *  working with String objects within Flex.
 *  You do not create instances of StringUtil;
 *  instead you call methods such as 
 *  the <code>StringUtil.substitute()</code> method.  
 * /
public class StringUtil
{
    ////////////////////////////////////////////////////////////////////////////////
//
//  ADOBE SYSTEMS INCORPORATED
//  Copyright 2005-2007 Adobe Systems Incorporated
//  All Rights Reserved.
//
//  NOTICE: Adobe permits you to use, modify, and distribute this file
//  in accordance with the terms of the license agreement accompanying it.
//
////////////////////////////////////////////////////////////////////////////////

import mx.core.mx_internal;

/**
 *  @private
 *  Version string for this class.
 * /
mx_internal static const VERSION:String = "3.3.0.4852";*/
    
    /*

    //--------------------------------------------------------------------------
    //
    //  Class methods
    //
    //--------------------------------------------------------------------------

    /**
     *  Removes all whitespace characters from the beginning and end
     *  of the specified string.
     *
     *  @param str The String whose whitespace should be trimmed. 
     *
     *  @return Updated String where whitespace was removed from the 
     *  beginning and end. 
     * /
    public static*/
    function trim$static(str) /*:String*/
    /*:String*/
    {
        if (str == null)  {
            return '';
        }
        
        var startIndex = /*:int*/
            0;
        while (StringUtil.isWhitespace(str.charAt(startIndex))) ++startIndex;
        var endIndex = /*:int*/
            str.length - 1;
        while (StringUtil.isWhitespace(str.charAt(endIndex))) --endIndex;
        if (endIndex >= startIndex)  {
            return str.slice(startIndex, endIndex + 1);
        }
        else  {
            return "";
        }
        
    }
    /*
    
    /**
     *  Removes all whitespace characters from the beginning and end
     *  of each element in an Array, where the Array is stored as a String. 
     *
     *  @param value The String whose whitespace should be trimmed. 
     *
     *  @param separator The String that delimits each Array element in the string.
     *
     *  @return Updated String where whitespace was removed from the 
     *  beginning and end of each element. 
     * /
    public static*/
    function trimArrayElements$static(value, /*:String*/
    delimiter) /*:String*/
    /*:String*/
    {
        if (value != "" && value != null) {
            var items = /*:Array*/
                value.split(delimiter);
            var len = /*:int*/
                items.length;
            for (var i = /*:int*/
                0; i < len; i++) {
                items[i] = StringUtil.trim(items[i]);
            }
            if (len > 0) {
                value = items.join(delimiter);
            }
        }
        return value;
    }
    /*

    /**
     *  Returns <code>true</code> if the specified string is
     *  a single space, tab, carriage return, newline, or formfeed character.
     *
     *  @param str The String that is is being queried. 
     *
     *  @return <code>true</code> if the specified string is
     *  a single space, tab, carriage return, newline, or formfeed character.
     * /
    public static*/
    function isWhitespace$static(character) /*:String*/
    /*:Boolean*/
    {
        switch (character) {
            case " ":
            case "\t":
            case "\r":
            case "\n":
            case "\f":
                return true;
            default:
                return false;
        }
    }
    /*

    /**
     *  Substitutes "{n}" tokens within the specified string
     *  with the respective arguments passed in.
     *
     *  @param str The string to make substitutions in.
     *  This string can contain special tokens of the form
     *  <code>{n}</code>, where <code>n</code> is a zero based index,
     *  that will be replaced with the additional parameters
     *  found at that index if specified.
     *
     *  @param rest Additional parameters that can be substituted
     *  in the <code>str</code> parameter at each <code>{n}</code>
     *  location, where <code>n</code> is an integer (zero based)
     *  index value into the array of values specified.
     *  If the first parameter is an array this array will be used as
     *  a parameter list.
     *  This allows reuse of this routine in other methods that want to
     *  use the ... rest signature.
     *  For example <pre>
     *     public function myTracer(str:String, ... rest):void
     *     { 
     *         label.text += StringUtil.substitute(str, rest) + "\n";
     *     } </pre>
     *
     *  @return New string with all of the <code>{n}</code> tokens
     *  replaced with the respective arguments specified.
     *
     *  @example
     *
     *  var str:String = "here is some info '{0}' and {1}";
     *  trace(StringUtil.substitute(str, 15.4, true));
     *
     *  // this will output the following string:
     *  // "here is some info '15.4' and true"
     * /
    public static*/
    function substitute$static(str) /*:String, ... rest*/
    /*:String*/
    {
        var rest = Array.prototype.slice.call(arguments, 1);
        if (str == null)  {
            return '';
        }
        
        // Replace all of the parameters in the msg string.
        var len = /*:uint*/
            rest.length;
        var args;
        /*:Array*/
        if (len == 1 && AS3.is(rest[0], Array)) {
            args = AS3.as(rest[0], Array);
            len = args.length;
        } else {
            args = rest;
        }
        for (var i = /*:int*/
            0; i < len; i++) {
            str = str.replace(new RegExp("\\{" + i + "\\}", "g"), String(args[i]));
        }
        return str;
    }
    /*
}

}

============================================== Jangaroo part ==============================================*/
    return {
        statics: {
            VERSION: "3.3.0.4852",
            trim: trim$static,
            trimArrayElements: trimArrayElements$static,
            isWhitespace: isWhitespace$static,
            substitute: substitute$static
        }
    };
});

